#--*-coding:utf-8-*--

class Cthulhu7th < DiceBot

  def initialize
    #$isDebug = true
    super
  end
  
  def gameName
    'クトゥルフ第7版'
  end
  
  def gameType
    "Cthulhu7th"
  end
  
  def prefixs
     ['CC\(\d+\)', 'CC.*', 'CBR\(\d+,\d+\)', 'FAR\(\d+\)' , 'FAR.*']
  end
  
  def getHelpMessage
    return <<INFO_MESSAGE_TEXT
・判定　CC(x)<=(目標値) で判定しクリティカル／極限の成功／困難な成功／通常成功／失敗／ファンブルの自動判定を行います。
・ボーナスダイス、ペナルティダイスがある1D100の目標値判定は、それぞれCCの後に()をつけ判定します。
　()の中はボーナスダイスなら1か2の数字、ペナルティダイスなら-1か-2の数字をつけます。0の場合はCCのみでも判定可能です。
例1）CC(2)<=30
　　　Cthulhu7th：（1D100<=50）→ 66,46,6 → 6 → 極限の成功
例2）CC(-1)<=75
　　　Cthulhu7th：（1D100<=75）→ 82,52 → 82 → 失敗

・組み合わせ判定　(CBR(x,y))
　目標値 x と y で％ロールを行い、成功したかどうかを表示します。
　例）CBR(50,20)
　　　Cthulhu7th : (1d100<=70,20) → 22[困難な成功,失敗] → 部分的成功

・連射（フルオート）判定　(FAR(w,x,y,z)) ※掃射の際には、余分な弾を使う弾数から減らしてから使用するようにしてください。
　wに使う弾数を、xに技能値を入れ、yに故障ナンバー、zにボーナス・ペナルティダイスの数字（-2～2）を入れることによって結果を算出します。ただし、wは1～100まで、x,zは1～100までの数字です。
　使う弾数が0になった時点で、そのラウンドの連射が終了します。弾が余った場合、結果に表示されます。
　ジャムが起きた場合、その段階で（ジャムの回の弾薬は消費せず）ロールを終了します。ジャムが起きず、ファンブルが起きた場合運の低い味方に命中します。
　ダメージ算出は命中数と貫通数から各自で計算するようにお願いします。FAR(w,x,y)の形でも判定可能です。
例1）FAR(50,80,98,-1)
　　　Cthulhu7th : ボーナス・ペナルティダイス[-1]
1回目: → 5, 15 → 極限の成功
2回目: → 50, 40, 30 → 通常成功
    難易度が困難な成功に変更
3回目: → 29, 89, 99 → 致命的失敗ジャム
8発が命中、4発が貫通、残弾34発

・追記（英語の訳）
　ファンブル：Fumble / 失敗：Failure / 通常成功：Regular success / 困難な成功：Hard success / 極限の成功：Extreme success / 決定的成功：Critical success
　ボーナスダイス：Bonus Dice / ペナルティダイス：Penalty Dice / 連射：Full Auto
INFO_MESSAGE_TEXT
  end
 
  def rollDiceCommand(command)
    case command
    when /CC/i
      return getCheckResult(command)  
    when /CBR/i
      return getCombineRoll(command)
    when /FAR/i
      return getFullAutoRoll(command)
    end
    
    return nil
  end
  
  
  def getCheckResult(command) 
    
    output = ""
    bonus_dice_count = 0  #ボーナス・ペナルティダイスの個数
    diff = 0
    
    
    if (/CC([-\d]+)?<=(\d+)/i =~ command)
      bonus_dice_count = $1.to_i
      diff = $2.to_i
    end
    
    
    return "エラー。目標値は1以上です。" if(diff <= 0)
    
    if((bonus_dice_count >= 3) or (bonus_dice_count <= -3))
      return "エラー。ボーナス・ペナルティダイスの値は-2～2です。"
    end
    
    
    output += "(1D100<=#{diff})"
    output += " ボーナス・ペナルティダイス[#{bonus_dice_count}]"
    
    units_digit = rollPercentD10
    total_list = getTotalLists(bonus_dice_count, units_digit)
    total = getTotal(total_list, bonus_dice_count)
    
    output += ' ＞ ' + "#{total_list.join(", ")}"
    
    output += " ＞ #{total}"
    output += " ＞ " + getCheckResultText(total, diff)
    
    return output
  end
  
  def rollPercentD10
    dice, = roll(1, 10)
    dice = 0 if(dice == 10)
    
    return dice
  end
  
  
  def getTotalLists(bonus_dice_count, units_digit)
    total_list = []
    
    tens_digit_count = 1 + bonus_dice_count.abs
    tens_digit_count.times do
      
      bonus = rollPercentD10
      total = (bonus * 10) + units_digit
      total = 100 if (total == 0)
      
      total_list.push(total)
    end
    
    return total_list
  end
  
  
  def getTotal(total_list, bonus_dice_count)
    total = if( bonus_dice_count >= 0 )
              total_list.min
            else
              total_list.max
            end
    
    return total
  end
  
  
  
  def getCheckResultText(total, diff, fumble_number = 0)
    
    result = ""
    
    if(total <= diff)
      
      if(total == 1)
        return "決定的成功"
      end
      
      if(total <= (diff / 5))
        return "極限の成功"
      end
      
      if(total <= (diff / 2))
        return "困難な成功"
      end
      
      return "通常成功"
      
    end
    
    
    if ((total >= 96) and (diff < 50))
      return "致命的失敗"
    end
    
    if ((total == 100) and (diff >= 50))
      return "致命的失敗"
    end
    
      
    if (((fumble_number == 1) and (total >=96)) and (diff >= 50))
      return "致命的失敗"
    end
    
    return "失敗"
  end
  
  
  def getCombineRoll(command)
    
    return nil unless(/CBR\((\d+),(\d+)\)/i =~ command)
    
    diff_1 = $1.to_i
    diff_2 = $2.to_i
    
    total, = roll(1, 100)
    
    result_1 = getCheckResultText(total, diff_1)
    result_2 = getCheckResultText(total, diff_2)
    
    successList = ["決定的成功", "極限の成功", "困難な成功", "通常成功"]
    
    succesCount = 0
    succesCount += 1 if successList.include?( result_1 )
    succesCount += 1 if successList.include?( result_2 )
    debug("succesCount", succesCount)
    
    rank = 
      if( succesCount >= 2 )
        "成功"
      elsif( succesCount == 1 )
        "部分的成功"
      else
        "失敗"
      end
    
    return "(1d100<=#{diff_1},#{diff_2}) ＞ #{total}[#{result_1},#{result_2}] ＞ #{rank}"
  end
  
  
  
  def getFullAutoRoll(command)
    
    return nil unless (/FAR\((-?\d+)(,(-?\d+))(,(-?\d+))(,(-?\d+))?\)/i =~ command)
    
    bullet_count = $1.to_i
    diff = $3.to_i
    broken_number = $5.to_i
    bonus_dice_count = ($7 || 0).to_i
    
    
    output = ""
    
    if (bullet_count > 100)  #最大で（8回*（PC技能値最大値/10））＝72発しか撃てないはずなので上限
      output += "\n弾薬が多すぎます。装填された弾薬を100発に変更します。\n"
      bullet_count = 100
    end
    
    return "弾薬は正の数です。" if (bullet_count < 0)  
    return "目標値は正の数です。" if (diff < 0)
    
    if (broken_number < 0)
      output += "\n故障ナンバーは正の数です。マイナス記号を外します。\n"
      broken_number = broken_number.abs
    end
    
    if ((bonus_dice_count > 2) or (bonus_dice_count < -2))
      return "\nエラー。ボーナス・ペナルティダイスの値は-2～2です。"
    end
    
    output += "ボーナス・ペナルティダイス[#{bonus_dice_count}]"
    
    loopCount = 1
    more_difficlty = 0
    dice_num = bonus_dice_count
    hit_bullet_count = 0
    impale_bullet_count = 0
    fumble_number = 0
    
    tithe_diff = diff / 10
    if ((diff >= 1) and (diff < 10))
      tithe_diff = 1  #技能値が9以下での最低値保障処理
    end
    
    impale_bullet_count_base = tithe_diff / 2.to_f
    
    hit_bullet_count_base = (diff / 10) / 2
    if ((diff >= 1) and (diff < 10))
      hit_bullet_count_base = 1  #技能値9以下での最低値保障
    end
    
    
    begin  #難易度変更用ループ
      begin  #ペナルティダイスを減らしながらロール用ループ
        
        break if (bullet_count <= 0)
        
        units_digit = rollPercentD10
        total_list = getTotalLists(dice_num, units_digit)
        total = getTotal(total_list, dice_num)
        
        output += "\n#{loopCount}回目:"
        
        output += ' ＞ ' + "#{total_list.join(", ")}"
        result = getCheckResultText(total, diff, fumble_number)
        output += ' ＞ ' + "#{result}"
        loopCount += 1
        dice_num -= 1
        
        
        isBroken = (total >= broken_number)
        
        if ( isBroken )
          output += "ジャム"
          break
        end
        
        
        #成功と貫通を分けるための処理
        successList, impaleBulletList = getSuccessListImpaleBulletList(more_difficlty)
        
        isSuccess = successList.include?( result ) 
        isImpale = impaleBulletList.include?( result ) 
        
        
        if ( not isLastBulletTurn(bullet_count, tithe_diff) )
          if isSuccess
            hit_bullet_count += hit_bullet_count_base #通常命中した弾数の計算
          end
          if isImpale  #貫通時
            hit_bullet_count += impale_bullet_count_base.floor
            impale_bullet_count += impale_bullet_count_base.ceil  #貫通した弾数の計算
          end
          
          bullet_count -= tithe_diff
          
        else
          if isSuccess
            hit_bullet_count = hitLastBullet(bullet_count, hit_bullet_count)
          end
          if isImpale
            halfbull = bullet_count / 2.to_f
            
            impale_bullet_count += halfbull.ceil
            hit_bullet_count += halfbull.floor
          end
          
          bullet_count = 0 #残弾が技能値/10以下の場合での残弾処理
        end
        
        
      end while (dice_num > -3)
      
      
      break if ( bullet_count <= 0 )
      break if ( isBroken )
      
      
      dice_num += 1
      more_difficlty += 1
      if (more_difficlty == 1)
        output += "\n    難易度が困難な成功に変更"
        fumble_number = 1  #成功が49以下の出目のみとなるため、ファンブルを上昇させる処理
      end
      if (more_difficlty == 2)
        output += "\n    難易度が極限の成功に変更"
      end
      if (more_difficlty == 3)
        output += "\n    難易度が決定的成功に変更"
      end
    end while (more_difficlty < 4 )
    
    
    output += "\n#{hit_bullet_count}発が命中、#{impale_bullet_count}発が貫通、残弾#{bullet_count}発"
    
    return output
  end
  
  
  def getSuccessListImpaleBulletList(more_difficlty)
    
    successList = []
    impaleBulletList = []
    
    case more_difficlty
    when 0
      successList = ["困難な成功", "通常成功"]
      impaleBulletList = ["決定的成功", "極限の成功"]
    when 1
      successList = ["困難な成功"]
      impaleBulletList = ["決定的成功", "極限の成功"]
    when 2
      successList = []
      impaleBulletList = ["決定的成功", "極限の成功"]
    when 3
      successList = ["決定的成功"]
      impaleBulletList = []
    end

    return successList, impaleBulletList
  end
  
  
  def isLastBulletTurn(bullet_count,tithe_diff)
    ((bullet_count - tithe_diff) < 0)
  end

  
  def hitLastBullet(bullet_count, hit_bullet_count)
    
    if (bullet_count == 1)
      #残弾1での最低値保障処理
      hit_bullet_count += 1
    else
      hit_bullet_count += (bullet_count / 2.to_f).floor
    end
    
    return hit_bullet_count
  end
  
  
end

